//
//  VdoPlayerPlatformView.swift
//  vdocipher_flutter
//
//  Created by Manish Gupta on 10/01/22.
//

import AVKit
import VdoFramework

class VdoNativeView: NSObject, FlutterPlatformView {
    private var _view: ObservableView
    private var methodChannel : FlutterMethodChannel?
    private var eventChannel : FlutterEventChannel?
    private var asset: VdoAsset?
    private var player: AVPlayer?
    private var playerLayer : AVPlayerLayer?
    private var loadResult : FlutterResult?
    private var playerEventSink: FlutterEventSink?
    private var playWhenReady: Bool = true
    private var playerState: PlayerState = .unknown {
        didSet{
            guard let eventChannelSink = playerEventSink else {return}
            let event = ["name" : "onPlayerStateChanged", "playWhenReady" : self.playWhenReady, "playbackState": self.playerState.rawValue] as [String : Any]
            eventChannelSink(event)
        }
    }
    private var playerItemBufferEmptyObserver: NSKeyValueObservation?
    private var playerItemBufferKeepUpObserver: NSKeyValueObservation?
    private var playerItemBufferFullObserver: NSKeyValueObservation?
    private var availableTracks: [[String: Any]] = [[String: Any]]()
    private var selectedTracks: [[String: Any]] = [[String: Any]]()
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger?
    ) {
        _view = ObservableView()
        super.init()
        VdoCipher.setPlaybackDelegate(delegate: self)
        _view.handler = { needsLayout in
            if needsLayout == true
            {
                self.playerLayer?.frame = self._view.bounds
                self.playerLayer?.setNeedsLayout()
            }
        }
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(videoDidEnd),
                                               name: NSNotification.Name.AVPlayerItemDidPlayToEndTime,
                                               object: nil)
        NotificationCenter.default.addObserver(self,
                           selector: #selector(handleInterruption),
                           name: AVAudioSession.interruptionNotification,
                           object: AVAudioSession.sharedInstance())
        // This is where we need to put in Method Channel Code
//        print("Arguements are \(String(describing: args)) View Identifier is \(viewId) Binary messenger is \(String(describing: messenger))")
        guard let messenger = messenger else {
            return
        }
        methodChannel = FlutterMethodChannel.init(name: "com.vdocipher.flutter.vdoplayerview_\(viewId)",
                                                  binaryMessenger: messenger)
        eventChannel = FlutterEventChannel.init(name: "com.vdocipher.flutter.vdoplayerview.events_\(viewId)",
                                                binaryMessenger: messenger)
        eventChannel?.setStreamHandler(self)
        methodChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            // Note: this method is invoked on the UI thread.
//            print("method channel call is \(call.method) and arguements are \(String(describing: call.arguments))")
            switch (call.method) {
            case "load":
                do{
                    guard let arguments = call.arguments as? [String: Any] else {
                        return
                    }
                    let restore = arguments["restore"] as? Bool
                    if restore == true
                    {
                        if self.player != nil
                        {
                            result(MessageUtils.loadResultMap(asset:self.asset, playWhenReady: self.playWhenReady))
                            PlayerHandler.sharedInstance.shouldResume = false
                            self.player?.play()
                            return
                        }
                        guard let asset = PlayerHandler.sharedInstance.asset, let otp = PlayerHandler.sharedInstance.otp, let playbackInfo = PlayerHandler.sharedInstance.playbackInfo else {
                            result("error message: Asset meta-data load failed! code: 3009")
                            return
                        }    //Need to send error back to flutter in case of return
                        self.asset = asset
                        result(MessageUtils.loadResultMap(asset:self.asset, playWhenReady: self.playWhenReady))
                        asset.playOnline(otp: otp, playbackInfo: playbackInfo)
                    }
                    else
                    {
                        guard let embedInfo = arguments["embedInfo"] as? [String: Any],
                              let otp = embedInfo["otp"] as? String,
                              let playbackInfo = embedInfo["playbackInfo"] as? String,
                              let data = Data(base64Encoded: playbackInfo),
                              let videoInfo = try? JSONSerialization.jsonObject(with: data, options: []) as? [String: Any],
                              let videoId = videoInfo["videoId"] as? String
                        else {
                            return
                        }
                        self.loadResult = result
//                        print("videoId is \(videoId), playbackInfo is \(playbackInfo) and otp is \(otp)")

                        VdoAsset.createAsset(videoId: videoId) { asset, displayableError  in
                            if let asset = asset
                            {
                                self.asset = asset
                                PlayerHandler.sharedInstance.asset = asset
                                PlayerHandler.sharedInstance.otp = otp
                                PlayerHandler.sharedInstance.playbackInfo = playbackInfo
                                PlayerHandler.sharedInstance.shouldResume = false
                                if let loadResult = self.loadResult
                                {
                                    loadResult(MessageUtils.loadResultMap(asset:self.asset, playWhenReady: self.playWhenReady))
                                    self.loadResult = nil
                                }
                                DispatchQueue.main.async {
                                    asset.playOnline(otp: otp, playbackInfo: playbackInfo)
                                }
                            }
                            else if let displayableError = displayableError
                            {
                                if let loadResult = self.loadResult
                                {
                                    loadResult(MessageUtils.loadResultMap(asset:self.asset, playWhenReady: false))
                                    self.loadResult = nil
                                }
                                result("error message: " + displayableError.message + " code: \(displayableError.code)")
                            }
                        }
                    }
                }
            case "setPlayWhenReady":
                do{
                    if call.arguments as? Int == 0
                    {
                        self.playWhenReady = false
                        self.player?.pause()
                    }
                    else
                    {
                        self.playWhenReady = true
                        self.player?.play()
                    }
                }
            case "getPlayWhenReady":
                do{
                    result(self.playerState == .ready ? self.playWhenReady : false)
                }
            case "getPlaybackAndBufferedPositions":
                do{
                    let currentTime = Int(self.player?.currentItem?.currentTime().seconds.rounded( .up) ?? 0.0 ) * 1000
                    result(["position": currentTime, "bufferedPosition": currentTime])
                }
            case "getDuration":
                do{
                    result(Int(self.player?.currentItem?.duration.seconds.rounded( .up) ?? 0.0 ) * 1000)
                }
            case "seek":
                do{
                    guard let currentTime = call.arguments as? Int else {return}
                    let currentRate = self.player?.rate
                    self.player?.seek(to: CMTime(value: CMTimeValue(currentTime), timescale: 1000))
                    self.player?.rate = currentRate ?? 1.0
                    result(nil)
                }
            case "stop":
                do{
                    
                }
            case "packUp":
                do{
                    PlayerHandler.sharedInstance.resumption = self.player?.currentTime()
                    PlayerHandler.sharedInstance.shouldResume = true
                    self.player?.pause()
                }
            case "setPlaybackSpeed":
                do{
                    guard let speed = call.arguments as? Double else {return}
                    self.player?.rate = Float(speed)
                }
            case "setAdaptive":
                do{

                }
            case "setAudioTrack":
                do{

                }
            case "setVideoTrack":
                do{

                }
            case "setSubtitleLanguage":
                do{
                    guard let group = self.player?.currentItem?.asset.mediaSelectionGroup(forMediaCharacteristic: .legible) else {
                        return
                    }
                    if let lanCode = call.arguments as? String {
                        let option = group.options.first { option in
                            option.extendedLanguageTag == lanCode
                        }
                        DispatchQueue.main.async {
                            self.player?.currentItem?.select(option, in: group)
                        }
                        self.selectedTracks.removeAll()
                        if let selectedSubtitle = self.availableTracks.first(where: { track in
                            track["language"] as! String == lanCode as String
                        }) {
                            self.selectedTracks.append(selectedSubtitle)
                        }
                        
                        guard let eventChannelSink = self.playerEventSink else {return}
                        let event = ["name": "onTracksChanged", "available": self.availableTracks, "selected": self.selectedTracks] as [String : Any]
                        eventChannelSink(event)
                    } else {
                        DispatchQueue.main.async {
                            self.player?.currentItem?.select(nil, in: group)
                        }
                        self.selectedTracks.removeAll()
                        guard let eventChannelSink = self.playerEventSink else {return}
                        let event = ["name": "onTracksChanged", "available": self.availableTracks, "selected": self.selectedTracks] as [String : Any]
                        eventChannelSink(event)
                    }
                }
            default:
                do{

                }
            }
        })
        methodChannel?.invokeMethod("onPlatformPlayerCreated", arguments: viewId)
    }
    
    @objc func handleInterruption(notification: Notification) {
        //        guard let eventChannelSink = playerEventSink else {return}
        //        let event = ["name" : "playbackSessionInterrupted"]
        //        eventChannelSink(event)
        guard let userInfo = notification.userInfo,
              let typeValue = userInfo[AVAudioSessionInterruptionTypeKey] as? UInt,
              let type = AVAudioSession.InterruptionType(rawValue: typeValue) else {
                  return
              }
        
        // Switch over the interruption type.
        switch type {
            
        case .began:
            do{
            // An interruption began. Update the UI as necessary.
            }
        case .ended:
            do{
                // An interruption ended. Resume playback, if appropriate.
                guard let optionsValue = userInfo[AVAudioSessionInterruptionOptionKey] as? UInt else { return }
                let options = AVAudioSession.InterruptionOptions(rawValue: optionsValue)
                if options.contains(.shouldResume) {
                    // An interruption ended. Resume playback.
                    self.player?.play()
                } else {
                    // An interruption ended. Don't resume playback.
                }
            }
        default: ()
        }
    }
    
    func addObserversToPlayer()
    {
        playerItemBufferEmptyObserver = self.player?.currentItem?.observe(\AVPlayerItem.isPlaybackBufferEmpty, options: [.new]) { [weak self] (_, _) in
            guard let self = self else { return }
            self.playerState = .buffering
        }
        
        playerItemBufferKeepUpObserver = self.player?.currentItem?.observe(\AVPlayerItem.isPlaybackLikelyToKeepUp, options: [.new]) { [weak self] (_, _) in
            guard let self = self else { return }
            self.playerState = .ready
        }
        
        playerItemBufferFullObserver = self.player?.currentItem?.observe(\AVPlayerItem.isPlaybackBufferFull, options: [.new]) { [weak self] (_, _) in
            guard let self = self else { return }
            self.playerState = .ready
        }
    }
    
    func removeObservers()
    {
        playerItemBufferEmptyObserver?.invalidate()
        playerItemBufferEmptyObserver = nil
            
        playerItemBufferKeepUpObserver?.invalidate()
        playerItemBufferKeepUpObserver = nil
            
        playerItemBufferFullObserver?.invalidate()
        playerItemBufferFullObserver = nil
    }
    
    @objc func videoDidEnd(notification: NSNotification) {
        guard let eventChannelSink = playerEventSink else {return}
        let event = ["name" : "onMediaEnded"]
        eventChannelSink(event)
        self.playerState = .ended
    }
    
    func view() -> UIView {
        return _view
    }
    
    func createNativeView(view _view: UIView){
        _view.backgroundColor = UIColor.blue
        let nativeLabel = UILabel()
        nativeLabel.text = "Native text from iOS"
        nativeLabel.textColor = UIColor.white
        nativeLabel.textAlignment = .center
        nativeLabel.frame = CGRect(x: 0, y: 0, width: 180, height: 48.0)
        _view.addSubview(nativeLabel)
    }
    
    deinit{
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name.AVPlayerItemDidPlayToEndTime,
                                                  object: nil)
        NotificationCenter.default.removeObserver(self,
                                                  name: AVAudioSession.interruptionNotification,
                                                  object: AVAudioSession.sharedInstance())
        removeObservers()
    }
}

extension VdoNativeView : AssetPlaybackDelegate {
    func streamLoadError(error: VdoError) {
        guard let eventChannelSink = playerEventSink else {return}
        eventChannelSink(FlutterError(code: "\(error.code)",
                                      message: error.message,
                                      details: nil))
    }
    
    func streamPlaybackManager(playerReadyToPlay player: AVPlayer) {
        // player is ready to play
//        print("iOS Native Framework - playerReadyToPlay")
        self.setPlay()
    }
    
    func streamPlaybackManager(playerCurrentItemDidChange player: AVPlayer) {
        // player current item has changed
//        print("iOS Native Framework - playerCurrentItemDidChange")
        playerLayer?.removeFromSuperlayer()
        self.removeObservers()
        self.player = player
        self.addObserversToPlayer()
        playerLayer = AVPlayerLayer(player: player)
        playerLayer?.frame = _view.bounds
        _view.layer.addSublayer(playerLayer!)
        
        self.availableTracks.removeAll()
        self.selectedTracks.removeAll()
        
        guard player.currentItem?.asset.availableMediaCharacteristicsWithMediaSelectionOptions.contains(where: {$0 == .legible}) == true,
              let group = player.currentItem?.asset.mediaSelectionGroup(forMediaCharacteristic: .legible) else {
            guard let eventChannelSink = playerEventSink else {return}
            let event = ["name" : "onTracksChanged", "available" : availableTracks] as [String : Any]
            eventChannelSink(event)
            return
        }
        
        var id = 0
        for option in group.options {
            guard let lanCode = option.extendedLanguageTag else {continue}
            availableTracks.append(["id": id, "type": "subtitle", "language": lanCode])
            id+=1
        }
        
        if let preferredSubtitleMediaOption = player.currentItem?.asset.preferredMediaSelection.selectedMediaOption(in: group), let lanCode = preferredSubtitleMediaOption.extendedLanguageTag, let selectedSubtitle = availableTracks.first(where: { track in
            track["language"] as! String == lanCode as String
        }) {
            selectedTracks.append(selectedSubtitle)
        }
        
        guard let eventChannelSink = playerEventSink else {return}
        let event = ["name": "onTracksChanged", "available": availableTracks, "selected": selectedTracks] as [String : Any]
        eventChannelSink(event)
    }
    
    func setPlay()
    {
//        print("setPlay called...")
        self.playerState = .ready
        if self.playWhenReady == true
        {
            if PlayerHandler.sharedInstance.shouldResume == true
            {
                guard let resumption = PlayerHandler.sharedInstance.resumption else {
                    player?.play()
                    return
                }
                PlayerHandler.sharedInstance.shouldResume = false
                player?.seek(to: resumption)
                player?.play()
                return
            }
            player?.play()
        }
    }
}

extension VdoNativeView : FlutterStreamHandler{
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        self.playerEventSink = events
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        self.playerEventSink = nil
        return nil
    }
}

private class MessageUtils{
    static func loadResultMap(asset : VdoAsset?, playWhenReady: Bool) -> [String: Any] {
        var result = [String : Any] ()
        if let asset = asset {
            result["didLoad"] = true
            result["mediaInfo"] = MessageUtils.loadMediaInfoMap(asset:asset)
            result["playWhenReady"] = playWhenReady
        }
        else
        {
            result["didLoad"] = false
        }
        return result
    }
    
    static func loadMediaInfoMap(asset : VdoAsset) -> [String: Any]  {
        var mediaInfoMap = [String : Any] ()
        mediaInfoMap["mediaId"] = asset.videoId
        mediaInfoMap["type"] = "fps"
        mediaInfoMap["title"] = asset.title
        mediaInfoMap["description"] = asset.description
        mediaInfoMap["duration"] = ( Int(asset.duration ?? 0.0) * 1000 )
        return mediaInfoMap
    }
}

private class ObservableView: UIView{
    
    var handler : ((Bool) -> Void)?
    
    override func layoutSubviews() {
        self.handler?(true)
    }
}

private enum PlayerState : String
{
    case buffering = "buffering",
         ready = "ready",
         ended = "ended",
         unknown = "unknown"
}

private class PlayerHandler : NSObject
{
    static let sharedInstance = PlayerHandler()
    var asset : VdoAsset?
    var otp : String?
    var playbackInfo: String?
    var resumption : CMTime?
    var shouldResume : Bool?
    private override init() {
        super.init()
    }
}
