import 'dart:io';

import 'package:flutter/material.dart';
import 'package:vdocipher_flutter/vdocipher_flutter.dart';

/// Widget for displaying video from your VdoCipher dashboard along with basic
/// video UI controls.
///
/// On being created, this widget creates a native platform video player and
/// provides a [VdoPlayerController] to control playback. The
/// [VdoPlayerController] reference is returned in the [onPlayerCreated] function
/// provided as an argument in the constructor.
class VdoPlayer extends StatefulWidget {
  final Function(VdoPlayerController controller) onPlayerCreated;
  final Function(bool isFullscreen)? onFullscreenChange;
  final Function(VdoError vdoError) onError;
  final EmbedInfo embedInfo;
  final double aspectRatio;
  final bool controls;

  VdoPlayer(
      {Key? key,
      required this.embedInfo,
      required this.onPlayerCreated,
      this.aspectRatio = 16 / 9,
      this.controls = true,
      this.onFullscreenChange,
      required this.onError})
      : super(key: key);

  @override
  _VdoPlayerState createState() => _VdoPlayerState();
}

class _VdoPlayerState extends State<VdoPlayer> {
  VdoPlayerController? _controller;
  VdoPlatformView? _vdoPlayerView;
  bool _isFullScreen = false;
  bool _isLoaded = false;

  _getVdoPlayerViewWidget() {
    if (_vdoPlayerView == null) {
      _vdoPlayerView = VdoPlatformView(
        key: UniqueKey(),
        onPlayerCreated: (controller) => _onPlayerCreated(controller),
        onPlayerLoaded: (isLoaded) => _onPlayerLoaded(isLoaded),
      );
    }

    return _vdoPlayerView;
  }

  _onFullscreenChange(isFullscreen) {
    setState(() {
      _isFullScreen = isFullscreen;
    });

    if (widget.onFullscreenChange != null) {
      widget.onFullscreenChange!(isFullscreen);
    }
  }

  _onPlayerLoaded(bool isLoaded) {
    setState(() {
      this._isLoaded = isLoaded;
    });
  }

  Stack _buildPlayerWithControls() {
    return Stack(
      children: <Widget>[
        Positioned.fill(
          child: _getVdoPlayerViewWidget(),
        ),
        _isLoaded || Platform.isIOS
            ? SizedBox.shrink()
            : Positioned.fill(
                child: Container(
                  color: Colors.black,
                ),
              ),
        _controller == null || !widget.controls
            ? SizedBox.shrink()
            : VdoControllerView(
                controller: _controller,
                onFullscreenChange: _onFullscreenChange,
                onError: widget.onError,
              )
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    Size screenSize = MediaQuery.of(context).size;

    return SafeArea(
        top: !_isFullScreen,
        left: !_isFullScreen,
        bottom: !_isFullScreen,
        right: !_isFullScreen,
        child: Container(
            width: screenSize.width,
            height: screenSize.height,
            color: Colors.black,
            child: AspectRatio(
              aspectRatio: widget.aspectRatio,
              child: _buildPlayerWithControls(),
            )));
  }

  _onPlayerCreated(VdoPlayerController controller) {
    _controller = controller;
    _controller!.load(widget.embedInfo);
    widget.onPlayerCreated(controller);
  }
}
